package com.qmth.teachcloud.mark.service.impl;

import com.alibaba.fastjson.JSON;
import com.qmth.teachcloud.common.bean.marking.MarkConfigItem;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.MarkQuestion;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.mark.SubjectiveStatus;
import com.qmth.teachcloud.common.service.SysUserService;
import com.qmth.teachcloud.common.service.TeachcloudCommonService;
import com.qmth.teachcloud.mark.dto.mark.MarkStudentVo;
import com.qmth.teachcloud.mark.dto.mark.ScoreItem;
import com.qmth.teachcloud.mark.dto.mark.SpecialTagDTO;
import com.qmth.teachcloud.mark.dto.mark.manage.*;
import com.qmth.teachcloud.mark.entity.*;
import com.qmth.teachcloud.mark.enums.MarkTaskStatus;
import com.qmth.teachcloud.mark.service.*;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

@Service
public class TaskServiceImpl implements TaskService {

    @Resource
    private MarkPaperService markPaperService;
    @Resource
    private MarkTaskService markTaskService;
    @Resource
    private MarkStudentService markStudentService;
    @Resource
    private MarkQuestionService markQuestionService;
    @Resource
    private MarkSubjectiveScoreService markSubjectiveScoreService;
    @Resource
    private TeachcloudCommonService teachcloudCommonService;
    @Resource
    private SysUserService sysUserService;

    @Override
    public Task build(MarkArbitrateHistory markArbitrateHistory, MarkQuestion markQuestion) {
        MarkStudent markStudent = markStudentService.getById(markArbitrateHistory.getStudentId());
        Task task = new Task();
        task.setStudentId(markArbitrateHistory.getStudentId());
        task.setTaskId(markArbitrateHistory.getId());
        task.setSecretNumber(markArbitrateHistory.getSecretNumber());
        task.setQuestionList(buildArbitrateStep(markQuestion, markArbitrateHistory));
        if (StringUtils.isNotBlank(markStudent.getObjectiveScoreList())) {
            task.setObjectiveScore(markStudent.getObjectiveScore());
        }
        if (StringUtils.isNotBlank(markStudent.getSheetPath())) {
            task.setSheetUrls(teachcloudCommonService.filePreview(markStudent.getSheetPathList(), SystemConstant.MARK_JPG_EXPIRE_MINUTE));
        }
        task.setMarkerTime(markArbitrateHistory.getUpdateTime());
        if (markArbitrateHistory.getTotalScore() != null) {
            task.setMarkerScore(markArbitrateHistory.getTotalScore());
        }
        if (StringUtils.isNotBlank(markQuestion.getPicList())) {
            task.setSliceConfig(JSON.parseArray(markQuestion.getPicList(), MarkConfigItem.class));
        }
        return task;
    }

    @Override
    public Task build(Long studentId) {
        MarkStudentVo markStudent = markStudentService.getMarkStudentVoByStudentId(studentId);
        Task task = new Task();
        if (markStudent != null) {
            task.setExamId(markStudent.getExamId());
            task.setStudentId(markStudent.getId());
            task.setSecretNumber(markStudent.getSecretNumber());
            task.setStudentCode(markStudent.getStudentCode());
            task.setStudentName(markStudent.getStudentName());
            task.setCourseCode(markStudent.getCourseCode());
            task.setCourseName(markStudent.getCourseName());
            task.setPaperNumber(markStudent.getPaperNumber());

            // 原图
            task.setSheetUrls(teachcloudCommonService.fileMarkPreview(markStudent.getSheetPathList()));

            if (!markStudent.getSubjectiveStatus().equals(SubjectiveStatus.UNMARK)) {
                task.setQuestionList(buildMarkStep(markStudent));
                task.setSpecialTagList(getMarkSpecialTagList(markStudent));
                task.setHeaderTagList(getHeaderTagList(markStudent));
            }
            if (StringUtils.isNotBlank(markStudent.getObjectiveScoreList())) {
                task.setObjectiveScore(markStudent.getObjectiveScore());
            }
            task.setMarkerScore(markStudent.getTotalScore());
            return task;
        }
        return task;
    }

    private List<MarkStepDTO> buildMarkStep(MarkStudent markStudent) {
        Long examId = markStudent.getExamId();
        String paperNumber = markStudent.getPaperNumber();
        List<MarkStepDTO> list = new LinkedList<MarkStepDTO>();
        List<MarkQuestion> sList = markQuestionService.listByExamIdAndPaperNumberAndObjective(examId, paperNumber, false);
        Set<Integer> mainList = markSubjectiveScoreService.listMainNumberByStudentIdAndUncalculate(markStudent.getId(),
                true);
        List<MarkSubjectiveScore> scoreList = markSubjectiveScoreService.listByStudentIdAndQuestionId(markStudent.getId(), null);
        for (int i = 0; i < scoreList.size(); i++) {
            MarkQuestion question = sList.get(i);
            String questionNumber = question.getQuestionNumber();
            MarkStepDTO step = buildStep(question);
            if (!scoreList.isEmpty() && scoreList.size() == sList.size()) {
                step.setScore(scoreList.get(i).getScore());
                if (mainList.contains(step.getMainNumber())) {
                    step.setUncalculate(true);
                }
            }
            // 增加阅卷轨迹列表获取
            List<MarkTask> markTaskList = markTaskService.listByStudentIdAndQuestionId(markStudent.getId(), question.getId());
            // 仲裁轨迹
            List<MarkHeaderTrack> headerTracks = new ArrayList<>();
            for (MarkTask markTask : markTaskList) {
                headerTracks.addAll(markTask.listHeaderTrack());
            }
            Map<Long, String> userNameMap = new HashMap<>();
            // 不管单评还是多评显示所有评卷员给分轨迹
            for (MarkTask markTask : markTaskList) {
                List<MarkTrack> tracks = markTask.listMarkerTrack();
                for (MarkTrack track : tracks) {
                    if (userNameMap.get(track.getUserId()) == null) {
                        SysUser user = sysUserService.getByUserId(track.getUserId());
                        userNameMap.put(track.getUserId(), user.getRealName() + "(" + user.getLoginName() + ")");
                    }
                    step.addTrack(replaceTrackFromHeaderTrack(track, headerTracks, userNameMap));
                }

                // 普通模式无轨迹数据。单独处理评卷员每题得分
                if (CollectionUtils.isEmpty(tracks) && markTask.getMarkerScore() != null) {
                    // 评卷员
                    if (markTask.getMarkerScore() != null) {
                        MarkerScoreDTO markerScoreDTO = new MarkerScoreDTO();
                        markerScoreDTO.setUserId(markTask.getUserId());
                        SysUser sysUser = sysUserService.getByUserId(markTask.getUserId());
                        if (sysUser != null) {
                            markerScoreDTO.setLoginName(sysUser.getLoginName());
                            markerScoreDTO.setUserName(sysUser.getRealName());
                        }
                        markerScoreDTO.setScore(markTask.getMarkerScore());
                        markerScoreDTO.setHeader(false);
                        step.addMarkerList(markerScoreDTO);
                    }
                    // 科组长
                    if (markTask.getHeaderScore() != null) {
                        if (markTask.getMarkerScore() != markTask.getHeaderScore()) {
                            MarkerScoreDTO markerScoreDTO = new MarkerScoreDTO();
                            markerScoreDTO.setUserId(markTask.getHeaderId());
                            SysUser sysUser = sysUserService.getByUserId(markTask.getHeaderId());
                            if (sysUser != null) {
                                markerScoreDTO.setLoginName(sysUser.getLoginName());
                                markerScoreDTO.setUserName(sysUser.getRealName());
                            }
                            markerScoreDTO.setScore(markTask.getHeaderScore());
                            markerScoreDTO.setHeader(true);
                            step.addMarkerList(markerScoreDTO);
                        }
                    }
                }
            }

            for (MarkHeaderTrack headerTrack : headerTracks) {
                TrackDTO trackDTO = new TrackDTO(headerTrack);
                SysUser sysUser = sysUserService.getByUserId(headerTrack.getUserId());
                if (sysUser != null) {
                    trackDTO.setUserId(sysUser.getId());
                    trackDTO.setUserName(sysUser.getRealName() + "(" + sysUser.getLoginName() + ")");
                }
                step.addHeaderTrack(trackDTO);
            }
            list.add(step);
        }
        return list;
    }

    private MarkerTrackDTO replaceTrackFromHeaderTrack(MarkTrack markTrack, List<MarkHeaderTrack> headerTracks, Map<Long, String> userNameMap) {
        MarkerTrackDTO markerTrackDTO = new MarkerTrackDTO(markTrack, userNameMap.get(markTrack.getUserId()));
//        if (CollectionUtils.isNotEmpty(headerTracks)) {
//            Optional<MarkHeaderTrack> optional = headerTracks.stream().filter(m -> m.getStudentId().equals(markTrack.getStudentId()) && m.getQuestionNumber().equals(markTrack.getQuestionNumber())).findFirst();
//            if (optional.isPresent()) {
//                MarkHeaderTrack markHeaderTrack = optional.get();
//                SysUser user = sysUserService.getByUserId(markHeaderTrack.getUserId());
//                markerTrackDTO = new MarkerTrackDTO(markHeaderTrack, user.getRealName() + "(" + user.getLoginName() + ")");
//            }
//        }
        return markerTrackDTO;
    }

    private List<MarkStepDTO> buildArbitrateStep(MarkQuestion markQuestion, MarkArbitrateHistory markArbitrateHistory) {
        List<MarkStepDTO> list = new LinkedList<>();

        List<MarkTask> markTaskList = markTaskService.listByStudentIdAndQuestionId(markArbitrateHistory.getStudentId(), markArbitrateHistory.getQuestionId());
        List<ScoreItem> sItems = new ArrayList<ScoreItem>();
        MarkStepDTO step = buildStep(markQuestion);
        if (markArbitrateHistory != null) {
            step.setScore(sItems.get(0).getScore());
            // 增加阅卷轨迹列表获取
            String questionNumber = markQuestion.getQuestionNumber();
            // todo 轨迹 2025-02-25
//            for (MarkHeaderTrack track : tracks) {
//                if (track.getQuestionNumber().equals(questionNumber)) {
//                    step.addTrack(new TrackDTO(track));
//                }
//            }
        }
        list.add(step);
        return list;
    }

    private MarkStepDTO buildStep(MarkQuestion question) {
        MarkStepDTO step = new MarkStepDTO();
        step.setMainNumber(question.getMainNumber());
        step.setSubNumber(String.valueOf(question.getSubNumber()));
        step.setQuestionId(question.getId());
        step.setTitle(question.getMainTitle());
        step.setDefaultScore(0d);
        step.setMaxScore(question.getTotalScore());
        step.setMinScore(0d);
        step.setIntervalScore(question.getIntervalScore());
        step.setUncalculate(false);
        return step;
    }

    @Override
    public Task build(MarkTask t) {
        Task task = new Task();
        task.setTaskId(t.getId());
        task.setQuestionId(t.getQuestionId());
        task.setSecretNumber(t.getSecretNumber());
        task.setMarkerTime(t.getMarkerTime());
        task.setPaperNumber(t.getPaperNumber());
        task.setMarkerScore(t.getMarkerScore());
        task.setRejected(t.getStatus().equals(MarkTaskStatus.REJECTED));
        task.setPrevious(t.getStatus().equals(MarkTaskStatus.MARKED));
        task.setStudentId(t.getStudentId());
        task.setStudentCode(t.getStudentCode());
        task.setQuestionList(buildMarkStep(t));
        task.setSpecialTagList(getMarkSpecialTagList(t));
        MarkStudent student = markStudentService.getById(t.getStudentId());
        task.setPaperType(student.getPaperType());
        task.setSheetUrls(teachcloudCommonService.fileMarkPreview(student.getSheetPathList()));
        MarkQuestion markQuestion = markQuestionService.getById(t.getQuestionId());
        task.setSliceConfig(markQuestion.getPictureConfigList());
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(t.getExamId(), t.getPaperNumber());
        task.setObjectiveScore(markPaper.getShowObjectScore() ? student.getObjectiveScore() : null);
        return task;
    }

    private List<MarkStepDTO> buildMarkStep(MarkTask task) {
        List<MarkStepDTO> list = new LinkedList<MarkStepDTO>();
        MarkQuestion markQuestion = markQuestionService.getById(task.getQuestionId());
        List<MarkTrack> tracks = task.listMarkerTrack();
        MarkStepDTO step = buildStep(markQuestion);
        if (task != null) {
            step.setScore(task.getMarkerScore());
            // 增加阅卷轨迹列表获取
            for (MarkTrack track : tracks) {
                step.addTrack(new TrackDTO(track));
            }
        }
        list.add(step);
        return list;
    }

    private SpecialTagDTO[] getHeaderTagList(MarkStudent student) {
        List<MarkTask> markTaskList = markTaskService.listByStudentId(student.getId());
        List<MarkHeaderTag> list = new ArrayList<>();
        for (MarkTask markTask : markTaskList) {
            list.addAll(markTask.listHeaderTag());
        }
        SpecialTagDTO[] specialTags = new SpecialTagDTO[list.size()];
        for (int i = 0; i < list.size(); i++) {
            specialTags[i] = new MarkerSpecialTagDTO(list.get(i).getUserId(), list.get(i).getGroupNumber(),
                    list.get(i));
        }
        return specialTags;
    }

    public SpecialTagDTO[] getMarkSpecialTagList(MarkTask t) {
        SpecialTagDTO[] specialTags = null;
        List<MarkSpecialTag> list = t.listMarkerTag();
        if (!list.isEmpty()) {
            specialTags = new SpecialTagDTO[list.size()];
            for (int i = 0; i < list.size(); i++) {
                SpecialTagDTO specialTagDTO = new SpecialTagDTO(list.get(i));
                specialTags[i] = specialTagDTO;
            }
        }
        return specialTags;
    }

    private SpecialTagDTO[] getMarkSpecialTagList(MarkStudent student) {
        List<SpecialTagDTO> list = new LinkedList<>();
        List<MarkTask> taskList = markTaskService.listByStudentId(student.getId());
        // 不管单评还是多评显示所有评卷员特殊标记
        for (MarkTask t : taskList) {
            List<MarkSpecialTag> tagList = t.listMarkerTag();
            for (MarkSpecialTag specialTag : tagList) {
                list.add(new MarkerSpecialTagDTO(t.getUserId(), t.getQuestionId(), specialTag));
            }
        }
        return list.toArray(new SpecialTagDTO[0]);
    }
}
